import React, { useRef, useEffect, useState } from "react";
import { Paper, makeStyles, Grid, IconButton, Icon, Typography, Fade } from "@material-ui/core";
import { DropzoneAreaStyle } from "./styles";
import { fileReader, sizeConverter, mimeType } from "./helper";
import PropTypes from "prop-types";
let func = () => {};
const App = ({
  onChange = func,
  type = "image",
  wrapperStyle = "",
  bannerStyle = "",
  BannerComponent,
  getProcessedData = func,
  inside = true,
  preview = true,
  PreviewWrapper = React.Fragment,
  bannerText = "Drag one or more files to this Drop Zone ...",
  uploadIcon = "cloud_upload",
  wrapperProps = {},
  limit = 1,
  limitErrorText = "you are trying to upload too many files ..."
}) => {
  const [state, setState] = useState({ files: [], imgUrl: [] });
  const ref = useRef(null);
  const inputRef = useRef(null);
  const styles = makeStyles(DropzoneAreaStyle)();
  useEffect(() => {
    "drag dragstart dragend dragover dragenter dragleave".split(" ").map(i =>
      ref.current.addEventListener(i, function(e) {
        e.preventDefault();
        e.stopPropagation();
      })
    );
    "dragover dragenter".split(" ").map(i =>
      ref.current.addEventListener(i, e => {
        if (
          e.dataTransfer.items[0].type.split("/")[0] !== type ||
          e.dataTransfer.items.length + state.files.length > limit
        ) {
          ref.current.classList.add(styles["dropzone_is_dragover_danger"]);
        } else {
          ref.current.classList.add(styles["dropzone_is_dragover"]);
        }
      })
    );
    "dragleave dragend".split(" ").map(i =>
      ref.current.addEventListener(i, e => {
        ref.current.classList.remove(styles["dropzone_is_dragover"], styles["dropzone_is_dragover_danger"]);
      })
    );
    ref.current.onClick = () => {};
  }, []);
  const onDrop = async e => {
    e.preventDefault();
    e.stopPropagation();
    ref.current.classList.remove(styles["dropzone_is_dragover"], styles["dropzone_is_dragover_danger"]);
    let droppedFiles = e.target.files || e.dataTransfer.files,
      { imgUrl, files } = state;
    if (droppedFiles.length + state.files.length > limit) {
      alert("Dosya Boyutu Çok Fazla");
      return;
    }
    for (let file of droppedFiles) {
      if (file.type.split("/")[0] !== type) {
        ref.current.classList.add(styles["dropzone_is_dragover_danger"]);
        return false;
      }
      files.push(file);
      imgUrl.push({ src: await fileReader(file), file });
    }
    setState({ ...state, files, imgUrl });
    getProcessedData(imgUrl);
    onChange(state);
  };
  const onDelete = index => {
    state.imgUrl = state.imgUrl.filter((i, k) => k !== index);
    state.files = state.files.filter((i, k) => k !== index);
    setState({ ...state });
    onChange(state);
  };
  return (
    <>
      <Paper
        elevation={2}
        {...wrapperProps}
        ref={ref}
        className={[styles.dropzone, wrapperStyle].join(" ")}
        onDrop={onDrop}
      >
        {BannerComponent ? (
          <BannerComponent triggerFileDialog={() => inputRef.current.click()} />
        ) : (
          <div className={[styles.dropzone_banner, bannerStyle].join(" ")}>
            <Typography variant="h5">{bannerText}</Typography>
            <IconButton color="primary" className={styles.dropzone_iconBtn} onClick={() => inputRef.current.click()}>
              <Icon className={styles.dropzone_icon}>{uploadIcon}</Icon>
            </IconButton>
          </div>
        )}
        {inside === true && preview === true && (
          <Grid container spacing={3}>
            {state.imgUrl.map((i, k) => (
              <Grid item xs={3} md={2} key={k}>
                <Fade in={true} timeout={{ enter: (k + 1) * 500, exit: (k + 1) * 500 }}>
                  {i.file.type.split("/")[0] === "image" ? (
                    <ImageFileWrapper item={i} onDelete={onDelete} index={k} />
                  ) : (
                    <OtherFileWrapper item={i} onDelete={onDelete} index={k} />
                  )}
                </Fade>
              </Grid>
            ))}
          </Grid>
        )}
      </Paper>
      <PreviewWrapper>
        {inside === false && preview === true && (
          <Grid container spacing={3}>
            {state.imgUrl.map((i, k) => (
              <Grid item xs={3} md={2} key={k}>
                <Fade in={true} timeout={{ enter: (k + 1) * 500, exit: (k + 1) * 500 }}>
                  {i.file.type.split("/")[0] === "image" ? (
                    <ImageFileWrapper item={i} onDelete={onDelete} index={k} />
                  ) : (
                    <OtherFileWrapper item={i} onDelete={onDelete} index={k} />
                  )}
                </Fade>
              </Grid>
            ))}
          </Grid>
        )}
      </PreviewWrapper>

      <input type="file" hidden ref={inputRef} onChange={onDrop} />
    </>
  );
};

App.propTypes = {
  BannerComponent: PropTypes.element,
  PreviewWrapper: PropTypes.element,
  bannerStyle: PropTypes.string,
  bannerText: PropTypes.string,
  getProcessedData: PropTypes.func,
  inside: PropTypes.bool,
  onChange: PropTypes.func,
  preview: PropTypes.bool,
  type: PropTypes.string,
  wrapperProps: PropTypes.object,
  uploadIcon: PropTypes.string,
  wrapperStyle: PropTypes.string,
  limit: PropTypes.number
};

const ImageFileWrapper = ({ item: i, onDelete, index: k }) => {
  const styles = makeStyles(DropzoneAreaStyle)();
  return (
    <div className={styles.relative}>
      <img src={i.src} className={styles.fullWidth} />
      <div className={styles.dropzone_overlay}>
        <Typography variant="caption" color="error">
          {sizeConverter(i.file.size)}
        </Typography>
        <IconButton color="secondary" onClick={() => onDelete(k)}>
          <Icon>delete</Icon>
        </IconButton>
      </div>
    </div>
  );
};
const OtherFileWrapper = ({ item: i, onDelete, index: k }) => {
  const styles = makeStyles(DropzoneAreaStyle)();
  return (
    <Paper className={[styles.dropzone_banner, styles.paddingArea].join(" ")}>
      <Icon fontSize="large">{mimeType()[i.file.type.split("/")[0]]}</Icon>
      <br />
      <div className={styles.dropzone_banner}>
        <Typography variant="caption" color="error">
          {sizeConverter(i.file.size)}
        </Typography>
        <IconButton color="secondary" onClick={() => onDelete(k)}>
          <Icon>delete</Icon>
        </IconButton>
      </div>
    </Paper>
  );
};
export default App;
